@@ -26,6 +26,10 @@
 
 #include "auth_mellon.h"
 
+#ifdef APLOG_USE_MODULE
+APLOG_USE_MODULE(auth_mellon);
+#endif
+
 /* This function is used to get the url of the current request.
  *
  * Parameters:
@@ -448,7 +452,7 @@ int am_check_permissions(request_rec *r, am_cache_entry_t *session)
                  match = !ap_regexec(ce->regex, value, 0, NULL, 0);
 
             } else if (ce->flags & (AM_COND_FLAG_SUB|AM_COND_FLAG_NC)) {
-                 match = (strcasestr(ce->str, value) != NULL);
+                 match = (ap_strcasestr(ce->str, value) != NULL);
 
             } else if (ce->flags & AM_COND_FLAG_SUB) {
                  match = (strstr(ce->str, value) != NULL);
@@ -1234,6 +1238,14 @@ int am_save_post(request_rec *r, const char **relay_state)
     apr_size_t written;
     apr_file_t *psf;
 
+    mod_cfg = am_get_mod_cfg(r->server);
+    if (mod_cfg->post_dir == NULL) {
+        ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r,
+                      "MellonPostReplay enabled but MellonPostDirectory not set "
+                      "-- cannot save post data");
+        return HTTP_INTERNAL_SERVER_ERROR;
+    }
+
     if (am_postdir_cleanup(r) != OK)
         return HTTP_INTERNAL_SERVER_ERROR;
 
@@ -1260,8 +1272,6 @@ int am_save_post(request_rec *r, const char **relay_state)
         charset = am_get_header_attr(r, content_type, NULL, "charset");
     }     
 
-    mod_cfg = am_get_mod_cfg(r->server);
-
     if ((psf_id = am_generate_id(r)) == NULL) {
         ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, "cannot generate id");
         return HTTP_INTERNAL_SERVER_ERROR;
@@ -1814,7 +1824,7 @@ tokenize(apr_pool_t *pool, const char *str, bool ignore_whitespace,
             if (*p != '\"') {
                 *error = apr_psprintf(pool,
                                       "unterminated string begining at "
-                                      "position %lu in \"%s\"",
+                                      "position %" APR_SIZE_T_FMT " in \"%s\"",
                                       start-str, str);
                 break;
             }
@@ -1840,7 +1850,7 @@ tokenize(apr_pool_t *pool, const char *str, bool ignore_whitespace,
         else {                  /* unrecognized token */
             *error = apr_psprintf(pool,
                                   "unknown token at "
-                                  "position %lu in string \"%s\"",
+                                  "position %" APR_SIZE_T_FMT " in string \"%s\"",
                                   p-str, str);
             break;
         }
@@ -1908,7 +1918,7 @@ parse_error_msg(apr_array_header_t *tokens, apr_size_t index)
         return "end of string";
     }
 
-    return apr_psprintf(tokens->pool, "\"%s\" at position %lu",
+    return apr_psprintf(tokens->pool, "\"%s\" at position %" APR_SIZE_T_FMT,
                         APR_ARRAY_IDX(tokens, index, Token).str,
                         APR_ARRAY_IDX(tokens, index, Token).offset);
 }
